body {
  background: #1a1e25;
  padding: 1rem 2rem;
}

.sermos_logo {
  margin: 2rem 0 0 1rem;
  float: right;
  width: 200px;
}

.markdown {
  color: #fff;
}

.swagger-ui a.nostyle {
  color: #f5a549;
}

.swagger-ui .opblock .opblock-section-header {
  background: hsla(0, 0%, 100%, 0.1);
}

.swagger-ui .info hgroup.main a {
  line-height: 3;
}

.swagger-ui .parameter__type {
  font-size: 12px;
  padding: 5px 0;
  font-family: monospace;
  font-weight: 600;
  color: #f5a549;
}

.swagger-ui .response-col_description__inner div.markdown {
  font-size: 15px;
  display: block;
  margin: 0;
  border-radius: 4px;
  background: #41444e;
  font-family: monospace;
  font-weight: 400;
  color: rgb(156, 148, 148);
}

.swagger-ui .response-col_description__inner div.markdown p {
  font-size: 15px;
  font-style: none;
  display: block;
  margin: 0;
  border-radius: 4px;
  background: #41444e;
  font-family: monospace;
  font-weight: 400;
  color: #f2f2f2;
}

.swagger-ui .opblock.opblock-post,
.swagger-ui .opblock.opblock-put,
.swagger-ui .opblock.opblock-delete,
.swagger-ui .opblock.opblock-get {
  border: none;
}

.swagger-ui .opblock.is-open .opblock-summary {
  border-bottom: none;
}

.swagger-ui .download-contents {
  position: absolute;
  bottom: 15px;
  right: 50px;
  cursor: pointer;
  background: #7d8293;
  -webkit-transition: all 0.6s ease-out;
  transition: all 0.6s ease-out;
  text-align: center;
  padding: 5px;
  border-radius: 4px;
  font-family: sans-serif;
  font-weight: 600;
  color: #fff;
  font-size: 14px;
  height: 26px;
  width: 77px;
}

.swagger-ui .download-contents:hover {
  background: #29b9f2;
}

.swagger-ui .scheme-container {
  background: transparent;
}

.swagger-ui .dialog-ux .modal-ux {
  background: rgba(27, 30, 36, 1);
  border: none;
}

.swagger-ui .auth-container input[type="text"] {
  background: white !important;
  color: black !important;
}
